﻿#load @"packages\FSharp.Charting.0.90.13\FSharp.Charting.fsx"
open FSharp.Charting
open System

// type providers for CSV, World Bank, etc. can be downloaded through NuGet. Documentation is here:
// http://fsharp.github.io/FSharp.Data/
#r @"packages\FSharp.Data.2.2.5\lib\net40\FSharp.Data.dll"
open FSharp.Data

// the data stays online, we just query for the exact answers we need
let wb = WorldBankData.GetDataContext()
type Country = WorldBankData.ServiceTypes.Country

// as you type '.' your options appear - this is why type providers are cool
wb.Countries.Serbia.CapitalCity

// We can easily combine with the charting library.
wb.Countries.Serbia.Indicators.``ATMs per 100,000 adults`` |> Chart.Line


wb.Regions.``European Union``.Countries
    |> Seq.map(fun c -> (c.Name, c.Indicators.``Adolescent fertility rate (births per 1,000 women ages 15-19)``.[2010]))
    |> Chart.Column 
    |> Chart.WithTitle ("Births per 1000 women ages 15-19")

let sample year = 
    wb.Regions.``European Union``.Countries
    |> Seq.where (fun country -> country.Indicators.``Population, total``.[year] > 10000000.0)
    |> Seq.toList
  
// what are the top 5 least populated countries in the EU? // (avg. 2000 - 2010)

let populationYears fromYear toYear (country: Country) =
    country.Indicators.``Population, total``
    |> Seq.filter (fun (year, value) -> year >= fromYear && year <= toYear)

wb.Regions.``European Union``.Countries
    |> Seq.map (fun c -> (c.Name, populationYears 2001 2010 c |> Seq.map snd |> Seq.average))
    |> Seq.sortBy (fun (name, avgPopulation) -> avgPopulation)
    |> Seq.take 5
    |> Seq.toList

let plotPopulation (country: Country) = 
    country
        |> populationYears 1960 2014
        |> fun data -> Chart.Line (data, Name = country.Name)
        |> Chart.WithXAxis (MinorTickMark = ChartTypes.TickMark(Enabled = true, Interval = 5.))


[wb.Countries.``Macedonia, FYR``; wb.Countries.Serbia; wb.Countries.Montenegro] 
    |> List.map plotPopulation 
    |> Chart.Combine
    |> Chart.WithLegend (Docking = ChartTypes.Docking.Left) // we need the brackets for non-F# methods


let countryNames = 2010 |> sample |> Seq.map (fun c -> c.Name)

let dataWB =
    [
        "Fertility_rate_per_1k", [ for c in sample 2008 -> c.Indicators.``Adolescent fertility rate (births per 1,000 women ages 15-19)``.[2008] ]
        "Youth_employment_%",    [ for c in sample 2008 -> c.Indicators.``Population ages 65 and above (% of total)``.[2008] ]
        "GDP_per_capita",        [ for c in sample 2008 -> c.Indicators.``GDP per capita growth (annual %)``.[2008] ]
        "Female_employers_%",    [ for c in sample 2008 -> c.Indicators.``Employers, female (% of employment)``.[2008] ]
    ] // can we do this without repeating "2008" so many times?

// A pretty fancy way to plot all pairs of variables against each other on scatter charts.
//
let plotDataFrames df labels =  
        let rec getPairs =
            function
            | [] | [_] -> []
            | head::tail -> 
                [ for e in tail do yield head, e 
                  yield! getPairs tail ]
        let getNamedPairs data = 
            let names, data = data |> List.unzip
            let pairs = data |> getPairs
            let namePairs = names |> getPairs
            List.zip namePairs pairs
        let combineCharts (charts : ChartTypes.GenericChart list) =
            let list1, list2 = charts |> List.mapi (fun i x -> (i, x)) |> List.partition (fun (i, _) -> i % 2 = 0)
            [list1 |> Seq.map snd;list2 |> Seq.map snd] |> List.map (Chart.Columns) |> Chart.Rows
        getNamedPairs df
        |> List.map (fun ((n1, n2),(d1, d2)) -> 
            let name = n1 + " -- " + n2
            let data = Seq.zip d1 d2
            Chart.Point(data, name, name,XTitle=n1,YTitle=n2,Labels=labels,MarkerSize=10))
        |> combineCharts
    
// Let's try it out.
(dataWB,countryNames) ||> plotDataFrames

// What's happening with Italy?
wb.Countries.Italy.Indicators.``Employers, female (% of employment)``
|> Chart.Line



